//AudioRecorder Class

#include "AudioRecorder.h"
#include "AnsiString.h"

AudioRecorder::AudioRecorder(HWND hwnd){
	error=0;
	RunRecord=0;
	OwnerHwnd=hwnd;
	waveBlocks_REC = allocateBlocks(BLOCK_SIZE_REC, BLOCK_COUNT_REC);
	if(waveBlocks_REC==NULL){
		error=1;
		return;
	}
	waveActualBlock_REC=0;

	// set up the WAVEFORMATEX structure
	wfx.nSamplesPerSec = 44100; /* sample rate */
	wfx.wBitsPerSample = 16; /* sample size */
	wfx.nChannels= 1; /* channels*/
	wfx.cbSize = 0; /* size of _extra_ info */
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nBlockAlign = (wfx.wBitsPerSample * wfx.nChannels) >> 3;
	wfx.nAvgBytesPerSec = wfx.nBlockAlign * wfx.nSamplesPerSec;

	if(waveInOpen(&hWaveIn, WAVE_MAPPER, &wfx, (DWORD)hwnd, 0L, WAVE_FORMAT_DIRECT | CALLBACK_WINDOW)){
		ShowError("Failed to open waveform input device","ERROR opening AudioRecorder");
		error=1;
	}
	
	RECdata=new unsigned char[BLOCK_SIZE_REC];
}

AudioRecorder::~AudioRecorder(){
	if(RunRecord)StopRec();
	// Wait until finished recording
	int i;
	for(i=0;i<BLOCK_COUNT_REC;i++)
		do {Sleep(10);} while (waveInUnprepareHeader(hWaveIn, &waveBlocks_REC[i], sizeof(WAVEHDR))==WAVERR_STILLPLAYING);
	freeBlocks(waveBlocks_REC);
	waveInClose(hWaveIn);
	delete RECdata;
	RECdata=NULL;
	
}

WAVEHDR* AudioRecorder::allocateBlocks(int size, int count){
	unsigned char* buffer;
	int i;
	WAVEHDR* blocks;
	DWORD totalBufferSize = (size + sizeof(WAVEHDR)) * count;
	if((buffer = (unsigned char*)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, totalBufferSize)) == NULL) {
		ShowError("Memory allocation error","ERROR opening Audio");
		return NULL;
	}
	blocks = (WAVEHDR*)buffer;
	buffer += sizeof(WAVEHDR) * count;
	for(i = 0; i < count; i++) {
		blocks[i].dwBufferLength = size;
		blocks[i].lpData = (CHAR*)buffer;
		blocks[i].dwBytesRecorded=0;
		blocks[i].dwUser = 0L;
		blocks[i].dwFlags = 0L;
		blocks[i].dwLoops = 0L;
		buffer += size;
	}
	return blocks;
}

void AudioRecorder::freeBlocks(WAVEHDR* blockArray){
	HeapFree(GetProcessHeap(), 0, blockArray);
}

void AudioRecorder::StartRec(){
	// Insert wave input buffers
	int i;
	for(i=0;i<BLOCK_COUNT_REC;i++){
		waveInPrepareHeader(hWaveIn, &waveBlocks_REC[i], sizeof(WAVEHDR));
		if(waveInAddBuffer(hWaveIn, &waveBlocks_REC[i], sizeof(WAVEHDR))){
			ShowError("Failed to read block from device","ERROR StartRec");
			error=1;
			return;
		}
	}
	// Commence sampling input
	if(waveInStart(hWaveIn)){
		ShowError("Failed to start recording","ERROR StartRec");
		error=1;
		return;
	}
	RunRecord=1;
}

void AudioRecorder::StopRec(){
	RunRecord=0;
	if(waveInStop(hWaveIn)){
		ShowError("Failed to stop recording","ERROR StopRec");
		error=1;
		return;
	}
}

//Must be called on message WIM_DATA from owner window procedure
unsigned char* AudioRecorder::GetRecData(){
	memcpy(RECdata,waveBlocks_REC[waveActualBlock_REC].lpData,BLOCK_SIZE_REC);
	
	if(RunRecord){
		//if(waveBlocks_REC[waveActualBlock_REC]->dwFlags & WHDR_PREPARED)
		waveInUnprepareHeader(hWaveIn, &waveBlocks_REC[waveActualBlock_REC], sizeof(WAVEHDR));
		waveInPrepareHeader(hWaveIn, &waveBlocks_REC[waveActualBlock_REC], sizeof(WAVEHDR));
		if(waveInAddBuffer(hWaveIn, &waveBlocks_REC[waveActualBlock_REC], sizeof(WAVEHDR))){
			ShowError("Failed to read block from device","ERROR GetRecData");
			error=1;
			return NULL;
		}
		waveActualBlock_REC++;
		waveActualBlock_REC&=0x1;
	}
	
	return RECdata;
}
