//Canvas Class
#ifndef CanvasClassH
#define CanvasClassH


#include "AnsiString.h"
#include "properties.h"

//class CanvasClass;
class PixelData;

class PixelDataRow{
  public:
    HDC* OwnerHDC;
    int xIndex;
    int yindex;
    
    int PixelDot;
    property<PixelDataRow, int> Pixel;
    void SetPixels(const int & pix){
        PixelDot=pix;
        SetPixel(*OwnerHDC,xIndex,yindex,RGB( (pix&0xFF),(pix&0xFF00)>>8,(pix&0xFF0000)>>16 ));
    }
    const int & GetPixels() const{
        int* pixd=(int*)&PixelDot;
        *pixd=GetPixel(*OwnerHDC,xIndex,yindex);
        return PixelDot;
    }
    int & GetRefPixel(){
        return PixelDot;
    }

    PixelDataRow():
    Pixel(this, &PixelDataRow::GetPixels, &PixelDataRow::SetPixels, &PixelDataRow::GetRefPixel){
      //Pixel=0;
    }
    ~PixelDataRow(){
    }
    property<PixelDataRow, int>& operator[](int yIndex){
      yindex=yIndex;
      return Pixel;
    }
};

class PixelData{
  public:
    PixelDataRow PixRow;
    PixelData(){
    }
    ~PixelData(){
    }
    PixelDataRow& operator[](int nIndex){
      PixRow.xIndex=nIndex;
      return PixRow;
    }
};

class BrushClass{
  public:
    BrushClass(){Color=0xFFFFFF;}
    ~BrushClass(){}
    int Color;
    void DrawFillRect(HDC hDC, RECT *lprc, HBRUSH hbr){
        FillRect(hDC,lprc,hbr);
    }
};

class PenClass{
  public:
    PenClass(){Color=0;}
    ~PenClass(){}
    int Color;
    void LineToDraw(HDC hdc,int nXEnd,int nYEnd){
        LineTo(hdc,nXEnd,nYEnd);
    }
};

class CanvasClass{
    void Init();
  public:
    HDC		Mhdc;
    HBITMAP Mhbmp;
    HBRUSH hBrush;
    HPEN hPen;
    HWND OwnerHwnd;        //The owner window
    int STX,STY;           //The (0,0) point of the canvas in the window
    int WID, HEI;
    int CurX,CurY;         //Actual Cursor positions
    bool EnableRefresh;
    //This is for standalone canvas with no LoadFromFile() option
    CanvasClass(HWND hwnd,int WIDTH,int HEIGHT);
	//This is for standalone canvas with no LoadFromFile() option; special cases X,Y,WID,HEI
	CanvasClass(HWND hwnd,int STARTX,int STARTY,int WIDTH,int HEIGHT,char mode);
    //This is for Bitmap image that can be loaded from file: LoadFromFile()
    CanvasClass(HWND hwnd,int STARTX,int STARTY,int WIDTH,int HEIGHT);
    ~CanvasClass();
    void SetPix(int pix,int xIndex,int yIndex);
    int GetPix(int xIndex,int yIndex);
    void Repaint();							//Called by Windows throught WM_PAINT message
	void Update();							//Called after drawing to the Canvas to update after
    void LoadFromFile(AnsiString filename);
    void LoadFromFile(char* filename);
    PixelData Pixels;
    BrushClass* Brush;
    PenClass* Pen;
    void FillRect(RECT r);
    void MoveTo(int x,int y);
    void LineTo(int x,int y);
};


class BitmapClass{
  public:
    CanvasClass* Canvas;
    BitmapClass(){}
    ~BitmapClass(){}
};

class PictureClass{
  public:
    BitmapClass* Bitmap;
    PictureClass(){
        Bitmap=new BitmapClass();
    }
    ~PictureClass(){
        delete Bitmap;
    }
    void LoadFromFile(AnsiString filename){
        Bitmap->Canvas->LoadFromFile(filename);
    }
    void LoadFromFile(char* filename){
        Bitmap->Canvas->LoadFromFile(filename);
    }
};

class ImageClass{
  public:
    PictureClass* Picture;
    CanvasClass* Canvas;
    ImageClass(HWND hwnd,int STARTX,int STARTY,int WIDTH,int HEIGHT){
        Picture=new PictureClass();
        Canvas=new CanvasClass(hwnd,STARTX,STARTY,WIDTH,HEIGHT);
        Picture->Bitmap->Canvas=this->Canvas;
    }
    ~ImageClass(){
        delete Picture;Picture=NULL;
        delete Canvas;Canvas=NULL;
    }
    void Repaint(){
        Canvas->Repaint();
    }
};

#endif

/*
Canvas->Brush->Color=0xFFFFFF;
Canvas->FillRect(r);
Canvas->Pen->Color=0;
Canvas->MoveTo(x,0);
Canvas->LineTo(x,250);
Canvas->Pixels[x][y]=kep[x][y];

Image1->Picture->Bitmap->Canvas->Pixels[x][y]=kep[x][y];
Image1->Picture->LoadFromFile("D:\\temp\\Push-pull3.bmp");
Image1->Canvas->Pixels[ PlineDB[z]->end->X ][ PlineDB[z]->end->Y ]=0xFF0000;
*/
